﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;
using System.Threading.Tasks;
using System.Windows.Forms;

using YoutubeExplode;
using YoutubeExplode.Videos.Streams;

namespace PFUL_Video_Downloader
{
    public partial class mainForm : Form
    {
        FolderBrowserDialog folderbrowserDialog;
        string aboutMessage = " Version 1 \n Developed by Shouvik Sarkar \n Developed in 2024 ";
        string aboutTitle = " About PFUL Video Downloader ";
        string downloadingText = "Status: Downloading Video!";
        string downloadedText = "Status: Downloaded Video.";

        string videoSource, videoLink, downloadLocation;
        string[] sources =
        {
            "Youtube"
        };

        public mainForm()
        {
            InitializeComponent();

            folderbrowserDialog = new FolderBrowserDialog();
            folderbrowserDialog.ShowNewFolderButton = true;
        }

        private void selectLocButton_Click(object sender, EventArgs e)
        {
            DialogResult result = folderbrowserDialog.ShowDialog();
            if (result == DialogResult.OK)
            {
                locationTextBox.Text = folderbrowserDialog.SelectedPath;
            }
        }

        private void aboutToolStripMenuItem_Click(object sender, EventArgs e)
        {
            MessageBox.Show(aboutMessage, aboutTitle, MessageBoxButtons.OK, MessageBoxIcon.Information);
        }

        private async void downloadButton_Click(object sender, EventArgs e)
        {
            videoSource = sourceComboBox.GetItemText(sourceComboBox.SelectedItem);
            videoLink = linkTextBox.Text;
            downloadLocation = locationTextBox.Text;
        
            if (!sources.Contains(videoSource))
            {
                MessageBox.Show("Video source selected is invalid!", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            if (videoLink == "" || !Uri.IsWellFormedUriString(videoLink, UriKind.Absolute))
            {
                MessageBox.Show("Video link entered is invalid!", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            if (downloadLocation == "" || !System.IO.Directory.Exists(downloadLocation))
            {
                MessageBox.Show("Path selected is invalid!", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            if (videoSource == "Youtube")
            {
                try
                {
                    if (await downloadVideoFromYoutube(videoLink, downloadLocation))
                    {
                        MessageBox.Show("Video downloaded successfully!", "Success", MessageBoxButtons.OK, MessageBoxIcon.Information);
                        return;
                    }
                }
                catch (Exception ex)
                {
                    MessageBox.Show("Error occurred while downloading video!", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    return;
                }
                
            }

            MessageBox.Show("Error occurred while downloading video!", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
        }

        private async Task<bool> downloadVideoFromYoutube(string vidURL, string dowLoc)
        {
            var youtube = new YoutubeClient();
            var streamManifest = await youtube.Videos.Streams.GetManifestAsync(vidURL);
            var streamInfo = streamManifest.GetMuxedStreams().GetWithHighestVideoQuality();
            var stream = await youtube.Videos.Streams.GetAsync(streamInfo);
            statusToolStripLabel.Text = downloadingText;
            downloadButton.Enabled = false;
            await youtube.Videos.Streams.DownloadAsync(streamInfo, dowLoc + "\\" + $"video_{DateTimeOffset.UtcNow.ToUnixTimeSeconds()}.{streamInfo.Container}");
            statusToolStripLabel.Text = downloadedText;
            downloadButton.Enabled = true;

            return true;
        }
    }
}
